{{- /* Modified version of https://github.com/imfing/hextra/blob/v0.10.0/layouts/_partials/footer.html */ -}}
{{- /* This file overrides the footer partial of the theme */ -}}

{{- $enableFooterSwitches := .Scratch.Get "enableFooterSwitches" | default false -}}
{{- $displayThemeToggle := site.Params.theme.displayToggle | default true -}}
{{- $footerSwitchesVisible := and $enableFooterSwitches (or hugo.IsMultilingual $displayThemeToggle) -}}
{{- $copyrightSectionVisible := or (.Site.Params.footer.displayPoweredBy | default true) .Site.Params.footer.displayCopyright -}}

{{- $copyright := (T "copyright") | default "© 2024 Hextra." -}}
{{- $poweredBy := (T "poweredBy") | default "Powered by Hextra" -}}

<footer class="hextra-footer hx:bg-gray-100 hx:pb-[env(safe-area-inset-bottom)] hx:dark:bg-neutral-900 hx:print:bg-transparent">
  {{- if $footerSwitchesVisible -}}
    <div class="hx:mx-auto hx:flex hx:gap-2 hx:py-2 hx:px-4 hextra-max-footer-width">
      {{- partial "language-switch.html" (dict "context" .) -}}
      {{- with $displayThemeToggle }}{{ partial "theme-toggle.html" }}{{ end -}}
    </div>
    {{- if or hugo.IsMultilingual $displayThemeToggle -}}
      <hr class="hx:border-gray-200 hx:dark:border-neutral-800" />
    {{- end -}}
  {{- end -}}

  {{- if $copyrightSectionVisible -}}
    <div class="hextra-max-footer-width hx:mx-auto  hx:mx-auto hx:flex hx:items-center hx:justify-end hx:gap-2 hx:h-16 hx:px-6 hx:text-gray-600 hx:dark:text-gray-400 hx:md:justify-start">

      <div class="hx:flex hx:w-full hx:flex-col hx:items-center hx:sm:items-start" >

        {{- if .Site.Params.footer.displayCopyright }}
        <span class="hx:mr-2 hx:inline">{{ $copyright | markdownify }}</span>
        {{- end -}}

        {{- if (.Site.Params.footer.displayPoweredBy | default true) }}
        <span class="hx:mr-2 hx:inline">{{ template "theme-credit" $poweredBy }}</span>
        {{- end -}}

      </div>

      {{- range .Site.Params.footer.links -}}
      {{- $external := strings.HasPrefix .url "http" -}}
      <a title="{{ .title }}" href="{{ .url }}" {{ if $external }}target="_blank"{{ end }} class="social-media hx:text-sm hx:contrast-more:text-gray-700 hx:contrast-more:dark:text-gray-100 hx:relative -hx:ml-2 hx:whitespace-nowrap hx:p-1 hx:md:inline-block hx:text-gray-600 hx:hover:text-gray-800 hx:dark:text-gray-400 hx:dark:hover:text-gray-200">
        <span class="hx:text-center">
           {{- partial "utils/icon.html" (dict "name" .icon "attributes" "height=20") -}}
        </span>
      </a>
      {{- end -}}

    </div>
  {{- end -}}
</footer>

{{- define "theme-credit" -}}
<a class="hx:flex hx:text-sm hx:items-center hx:gap-1 hx:text-current" target="_blank" rel="noopener noreferrer" title="Hextra GitHub Homepage" href="https://github.com/imfing/hextra">
    <span>
      {{- . | markdownify -}}
      {{- if strings.Contains . "Hextra" -}}
        {{- partial "utils/icon.html" (dict "name" "hextra" "attributes" `height=1em class="hx:inline-block hx:ltr:ml-1 hx:rtl:mr-1 hx:align-[-2.5px]"`) -}}
      {{- end -}}
    </span>
</a>
{{- end -}}
