{{- /*
Embeds a file.

@param {string} file The path to the file.
@returns {string}

@example {{% golangci/embed file="path/to/file.txt" %}}
*/ -}}

{{- $file := .Get "file" -}}

{{ readFile $file | safeHTML}}
