{{- /*
This shortcode is used to create a filter bar.

@example {{< golangci/items/filter >}}{{< golangci/items/filter-badge content="Example">}}{{< golangci/items/filter >}}
*/ -}}

<div class="hx:mt-6">
  <span class="hx:inline-block hx:align-text-bottom icon" title="Filter">
    {{- partial "utils/icon" (dict "name" "filter" "attributes" "height=1.2em") -}}
  </span>
  {{ .Inner }}
</div>
