{{- /*
Displays the latest version of golangci-lint.
Based on the data file `data/version.json`.

@example {{< golangci/latest-version >}}
*/ -}}

{{- $v := index hugo.Data.version.version -}}
{{- if $v -}}
  {{- $v -}}
{{- else if .Page.GitInfo -}}
  {{- .Page.GitInfo.AbbreviatedCommit -}}
{{- else -}}
  devel
{{- end -}}
