{{- /*
Creates a code block with the output of the CLI.

@param {string} cmd The name of the command.
@param {string} section The name of the section inside the data file.

@example {{% golangci/cli-output %}}
@example {{% golangci/cli-output cmd="foo" %}}
@example {{% golangci/cli-output section="sectionName" cmd="foo bar" %}}
*/ -}}

{{- $cmdName := .Get "cmd" -}}
{{- $section := .Get "section" -}}

{{- $cliData := index hugo.Data.cli_help -}}

```console
{{ if $section -}}

$ golangci-lint {{ $cmdName }}

{{- else -}}
    {{- $section = print (or $cmdName "root") "Output" -}}

$ golangci-lint{{ if $cmdName }} {{ $cmdName }}{{ end }} -h

{{- end }}
{{ index $cliData $section | safeHTML }}
```
