{{- /*
Creates an exclusion presets configuration snippet.

@example {{% golangci/exclusion-presets-snippet %}}
*/ -}}

{{- $presets := index hugo.Data.exclusion_presets -}}

```yaml
linters:
  exclusions:
    presets:
{{- range $key, $values := $presets }}
      - {{ $key }}
{{- end }}
```

