{{- /*
Creates a code block with the contents of a configuration file section.

@param {string} section The name of the section inside the data file.
@returns {string}

@example {{% golangci/configuration-file-snippet section="sectionName" %}}
*/ -}}

{{- $sectionName := .Get "section" -}}

{{- $file := index hugo.Data.configuration_file -}}

```yaml
{{ index $file $sectionName | safeHTML }}
```
