{{- /*
Compares two versions.

@param {string} a Version A
@param {string} b Version B
@returns {boolean}

@example {{ partial "golangci/items/compare-versions" (dict "a" "v1.2.3" "b" "v1.2.4") }}
*/ -}}

{{- $aVersion := path.Dir (replace .a "." "/") -}}
{{- $bVersion := path.Dir (replace .b "." "/") -}}

{{- return (eq $aVersion $bVersion) -}}
