// Code generated by pkg/commands/internal/migrate/cloner/cloner.go. DO NOT EDIT.

package versiontwo

type Issues struct {
	MaxIssuesPerLinter *int  `yaml:"max-issues-per-linter,omitempty" toml:"max-issues-per-linter,multiline,omitempty"`
	MaxSameIssues      *int  `yaml:"max-same-issues,omitempty" toml:"max-same-issues,multiline,omitempty"`
	UniqByLine         *bool `yaml:"uniq-by-line,omitempty" toml:"uniq-by-line,multiline,omitempty"`

	DiffFromRevision  *string `yaml:"new-from-rev,omitempty" toml:"new-from-rev,multiline,omitempty"`
	DiffFromMergeBase *string `yaml:"new-from-merge-base,omitempty" toml:"new-from-merge-base,multiline,omitempty"`
	DiffPatchFilePath *string `yaml:"new-from-patch,omitempty" toml:"new-from-patch,multiline,omitempty"`
	WholeFiles        *bool   `yaml:"whole-files,omitempty" toml:"whole-files,multiline,omitempty"`
	Diff              *bool   `yaml:"new,omitempty" toml:"new,multiline,omitempty"`

	NeedFix *bool `yaml:"fix,omitempty" toml:"fix,multiline,omitempty"`
}
