// Code generated by pkg/commands/internal/migrate/cloner/cloner.go. DO NOT EDIT.

package versiontwo

type BaseRule struct {
	Linters    []string `yaml:"linters,omitempty" toml:"linters,multiline,omitempty"`
	Path       *string  `yaml:"path,omitempty" toml:"path,multiline,omitempty"`
	PathExcept *string  `yaml:"path-except,omitempty" toml:"path-except,multiline,omitempty"`
	Text       *string  `yaml:"text,omitempty" toml:"text,multiline,omitempty"`
	Source     *string  `yaml:"source,omitempty" toml:"source,multiline,omitempty"`

	InternalReference *string `yaml:"-,omitempty" toml:"-,multiline,omitempty"`
}
