//go:build go1.21

//golangcitest:args -Esloglint
//golangcitest:config_path testdata/sloglint_no_raw_keys.yml
package testdata

import "log/slog"

const foo = "foo"

func test() {
	slog.Info("msg", foo, 1)
	slog.Info("msg", "foo", 1)           // want `the "foo" key should be a constant`
	slog.Info("msg", slog.Int("foo", 1)) // want `the "foo" key should be a constant`
}
