// Code generated by pkg/commands/internal/migrate/cloner/cloner.go. DO NOT EDIT.

package versiontwo

type Linters struct {
	Default  *string  `yaml:"default,omitempty" toml:"default,multiline,omitempty"`
	Enable   []string `yaml:"enable,omitempty" toml:"enable,multiline,omitempty"`
	Disable  []string `yaml:"disable,omitempty" toml:"disable,multiline,omitempty"`
	FastOnly *bool    `yaml:"fast-only,omitempty" toml:"fast-only,multiline,omitempty"`

	Settings LintersSettings `yaml:"settings,omitempty" toml:"settings,multiline,omitempty"`

	Exclusions LinterExclusions `yaml:"exclusions,omitempty" toml:"exclusions,multiline,omitempty"`
}
