// Package testdata ...
//
//golangcitest:args -Estaticcheck
//golangcitest:config_path testdata/stylecheck.yml
//golangcitest:expected_exitcode 0
package testdata

func _(x int) {
	if x == 42 {

	}
}
