// Code generated by pkg/commands/internal/migrate/cloner/cloner.go. DO NOT EDIT.

package versiontwo

type LinterExclusions struct {
	Generated   *string       `yaml:"generated,omitempty" toml:"generated,multiline,omitempty"`
	WarnUnused  *bool         `yaml:"warn-unused,omitempty" toml:"warn-unused,multiline,omitempty"`
	Presets     []string      `yaml:"presets,omitempty" toml:"presets,multiline,omitempty"`
	Rules       []ExcludeRule `yaml:"rules,omitempty" toml:"rules,multiline,omitempty"`
	Paths       []string      `yaml:"paths,omitempty" toml:"paths,multiline,omitempty"`
	PathsExcept []string      `yaml:"paths-except,omitempty" toml:"paths-except,multiline,omitempty"`
}

type ExcludeRule struct {
	BaseRule `yaml:",inline"`
}
