:root {
  --primary-hue: 187deg;
  --primary-saturation: 54.4%;
  --primary-lightness: 57.8%;
}

/* Allow card subtitle with more than 3 lines */
.hx\:line-clamp-3.hextra-card-subtitle {
  -webkit-line-clamp: 30;
}

.author-cards {
    grid-template-columns: repeat(auto-fill, minmax(max(150px, calc((100% - 1rem * 4) / var(--hextra-cards-grid-cols))), 1fr));
}

.support-cards {
    grid-template-columns: repeat(auto-fill, minmax(max(200px, calc((100% - 1rem * 4) / var(--hextra-cards-grid-cols))), 1fr));
}

.support-cards .hextra-card-image {
    margin: 2rem;
}

.gl-hidden {
    display: none;
}

.hextra-nav-container a[title="Support us"] {
    transition: ease-in-out 0.2s;
}

.hextra-nav-container a[title="Support us"]:hover  {
    transform: scale(1.3, 1.3);
}

.hextra-footer .social-media {
    transition: ease-in-out 0.2s;
}

.hextra-footer .social-media :hover{
    transform: translateY(-2px);
}

.hextra-banner {
    background: #833AB4;
    background: linear-gradient(90deg,rgba(131, 58, 180, 1) 0%, rgba(253, 29, 29, 1) 50%, rgba(252, 176, 69, 1) 100%);
    /*color: var(--hx-color-slate-900);*/
}

.extra-banner-close-button {
    color: var(--hx-color-slate-50);
}
