// Code generated by pkg/commands/internal/migrate/cloner/cloner.go. DO NOT EDIT.

package versiontwo

type Formatters struct {
	Enable     []string            `yaml:"enable,omitempty" toml:"enable,multiline,omitempty"`
	Settings   FormatterSettings   `yaml:"settings,omitempty" toml:"settings,multiline,omitempty"`
	Exclusions FormatterExclusions `yaml:"exclusions,omitempty" toml:"exclusions,multiline,omitempty"`
}

type FormatterExclusions struct {
	Generated  *string  `yaml:"generated,omitempty" toml:"generated,multiline,omitempty"`
	Paths      []string `yaml:"paths,omitempty" toml:"paths,multiline,omitempty"`
	WarnUnused *bool    `yaml:"warn-unused,omitempty" toml:"warn-unused,multiline,omitempty"`
}
