// Code generated by pkg/commands/internal/migrate/cloner/cloner.go. DO NOT EDIT.

package versiontwo

type Severity struct {
	Default *string        `yaml:"default,omitempty" toml:"default,multiline,omitempty"`
	Rules   []SeverityRule `yaml:"rules,omitempty" toml:"rules,multiline,omitempty"`
}

type SeverityRule struct {
	BaseRule `yaml:",inline"`
	Severity *string `yaml:"severity,omitempty" toml:"severity,multiline,omitempty"`
}
