//golangcitest:args -Efunlen
//golangcitest:config_path testdata/funlen_ignore_comments.yml
package testdata

// want +1 "Function 'withComments' is too long"
func withComments() {
	// Comment 1
	// Comment 2
	// Comment 3
	// Comment 4
	// Comment 5
	// Comment 6
	// Comment 7
	// Comment 8
	// Comment 9
	// Comment 10
	// Comment 11
	// Comment 12
	// Comment 13
	// Comment 14
	// Comment 15
	// Comment 16
	// Comment 17
	// Comment 18
	// Comment 19
	// Comment 20
	// Comment 21
	// Comment 22
	// Comment 23
	// Comment 24
	// Comment 25
	// Comment 26
	// Comment 27
	// Comment 28
	// Comment 29
	// Comment 30
	// Comment 31
	// Comment 32
	// Comment 33
	// Comment 34
	// Comment 35
	// Comment 36
	// Comment 37
	// Comment 38
	// Comment 39
	// Comment 40
	// Comment 41
	// Comment 42
	// Comment 43
	// Comment 44
	// Comment 45
	// Comment 46
	// Comment 47
	// Comment 48
	// Comment 49
	// Comment 50
	// Comment 51
	// Comment 52
	// Comment 53
	// Comment 54
	// Comment 55
	// Comment 56
	// Comment 57
	// Comment 58
	// Comment 59
	// Comment 60
	print("Hello, world!")
}
