// Code generated by pkg/commands/internal/migrate/cloner/cloner.go. DO NOT EDIT.

package versiontwo

type Config struct {
	Version *string `yaml:"version,omitempty" toml:"version,multiline,omitempty"`

	Run Run `yaml:"run,omitempty" toml:"run,multiline,omitempty"`

	Output Output `yaml:"output,omitempty" toml:"output,multiline,omitempty"`

	Linters Linters `yaml:"linters,omitempty" toml:"linters,multiline,omitempty"`

	Issues   Issues   `yaml:"issues,omitempty" toml:"issues,multiline,omitempty"`
	Severity Severity `yaml:"severity,omitempty" toml:"severity,multiline,omitempty"`

	Formatters Formatters `yaml:"formatters,omitempty" toml:"formatters,multiline,omitempty"`
}
