// Package versionone contains a modified copy of v1 configuration.
// The structures are altered to use pointer on builtin types.
// The field version is added to enforce the detection of already migrated file.
package versionone
